
package parser;

import java.util.*;

public class Parser extends Syntax {
    private final boolean supportAssertion;
    private final boolean supportEnum;
    public Parser(boolean supportAssertion, boolean supportEnum) {
        this.supportAssertion = supportAssertion;
        this.supportEnum = supportEnum;
    }
    
    public static class Identifier extends Default.Token {
        public Identifier(Token token) {
            super(TOKEN_IDENTIFIER, token.getImage(), token.getOriginalImage(), token);
        }
    }
    
    protected LexicalAnalyzer createLexicalAnalyzer(String sourceName, CharSequence text, int tabStop) throws ParseException {
        return new Default.LexicalAnalyzer(sourceName, text, tabStop) {
            protected char nextChar() throws ParseException {
                return super.nextUnicodeEscapedChar();
            }
            public Token next() throws ParseException {
                Token result = super.next();
                if (result.getSymbolID() == TOKEN_IDENTIFIER || (!supportAssertion && result.getSymbolID() == TOKEN_ASSERT) || (!supportEnum && result.getSymbolID() == TOKEN_ENUM)) {
                    result = new Identifier(result);
                    String image = result.getImage();
                    int i = 0;
                    for (; i < image.length(); i++) {
                        char ch = image.charAt(i);
                        if (i == 0) {
                            if (!Character.isJavaIdentifierStart(ch))
                                break;
                        } else {
                            if (!Character.isJavaIdentifierPart(ch))
                                break;
                        }
                    }
                    if (i < image.length())
                        throw new ParseException("The " + i + "th letter \\u" + Integer.toHexString(image.charAt(i)) +  " of the token " + result + " cannot be used.", result);
                }
                return result;
            }
        };
    }
    
    private static Token toToken(Node node) {
        String image = getImage(node);
        int line = getLine(node);
        return new Default.Token(TOKEN_IDENTIFIER, image, image, null, -1, line, -1);
    }
    private static String getImage(Node node) {
        StringBuffer result = new StringBuffer();
        getImage(node, result);
        return result.toString();
    }
    private static void getImage(Node node, StringBuffer result) {
        if (node instanceof Token) {
            result.append(((Token) node).getImage());
        } else {
            Iterator it0 = node.getChildNodes().iterator();
            while (it0.hasNext()) {
                Node child = (Node) it0.next();
                getImage(child, result);
            }
        }
    }
    private static int getLine(Node node) {
        int result = -1;
        if (node instanceof Token) {
            return ((Token) node).getLine();
        } else {
            Iterator it0 = node.getChildNodes().iterator();
            while (it0.hasNext()) {
                Node child = (Node) it0.next();
                result = getLine(child);
                if (result != -1)
                    break;
            }
        }
        return result;
    }
    private static class InstanceInitializerReplacement extends Default.Node implements InstanceInitializer {
        public InstanceInitializerReplacement(Node node) {
            super(Collections.singletonList(toToken(node)));
        }
        public Block block() {
            return null;
        }
    }
    private static class StaticInitializerReplacement extends Default.Node implements StaticInitializer {
        public StaticInitializerReplacement(Node node) {
            super(Collections.singletonList(toToken(node)));
        }
        public Block block() {
            return null;
        }
    }
    /*protected Node createNode(int symbolID, NodeInitializationParameters parameters) throws ParseException {
        Node result = super.createNode(symbolID, parameters);
        if (result instanceof InstanceInitializer) {
            result = new InstanceInitializerReplacement(result);
        }
        if (result instanceof StaticInitializer) {
            result = new StaticInitializerReplacement(result);
        }
        return result;
    }*/
    
    public static void main(String[] args) {
        try {
            CompilationUnit root = new Parser(true, true).parseCompilationUnit(new java.io.File(args[0]), null);
            System.out.println(root);
        } catch(Exception x) {
            x.printStackTrace();
        }
    }
}

